# -------------------------------------------------------------------------------------
# flo::editor::Html
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden.
#
#    Description:
#
#      HTML Content component. Lets the user adds chunks of HTML to pages
#
# -------------------------------------------------------------------------------------
package flo::editor::Html;
use MKDoc::XML::XHTML::Linkifier;
use Unicode::Normalize;
use flo::Standard qw /general_config cgi path_info raw_path_info session table template current_document/;
use flo::Editor;
use flo::XHTMLFilter;
use lib::util::Command;
use MKDoc::Util::Text2HTML;
use MKDoc::Config;
use strict;
use MKDoc::Ouch;
use Petal::Entities;

use base qw /flo::Component
	     flo::editor::Mixin::compute_name
	     flo::editor::Mixin::normalize_name/;


sub preferred_extension { 'html' };


##
# $self->new;
# -----------
#   Returns a new flo::editor::Html object based on CGI input
##
sub new
{
    my $class = shift;
    $class    = ref $class || $class;
    my $self  = bless { @_ }, $class;
    
    my $cgi = $self->{cgi};
    my $param_name = $self->{param_name};
    
    if (defined $cgi)
    {
	if (defined $cgi->param ($param_name . "_add"))
	{
	    $self->{data} = "";
	}
	else
	{
	    $self->{data} = $cgi->param ($param_name);
	}
    }
    else
    {
	$self->{data} = "";
    }
    $self->{class} = 'html';
    return $self;
}


##
# $self->generate_xml;
# --------------------
#   Generates the XML for that component
##
sub generate_xml
{
    my $self = shift;
    my $data = $self->{data};
    Petal::Entities::decode_entities ($data);
    return $data;
}


##
# $self->parse_xml;
# -----------------
#   Parses the XML to reconstruct the component
##
sub parse_xml
{
    my $self = shift;
    my $data = shift;
    $self->{data} = $data;
}


sub as_xhtml { return "<div>" . shift()->html() . "</div>" }


##
# $self->html;
# ------------
#   Returns linkified HTML for that component
##
sub html
{
    my $self = shift;
    my $data = "<body>$self->{data}</body>";

    # normalizes the data to form C so that hyperlinking
    # works smoothly...
    $data = NFC ($data);
    # u_mark (\$data);
    
    my $tree = new MKDoc::XML::XHTML::Linkifier ( $data );
    $tree->trim_spaces;
    
    $tree->linkify (flo::Editor->links);
    my $res = $tree->text_nice;
    $res =~ s/.*\<body\>\n?//;
    $res =~ s/\n?\<\/body\>.*//;
    return $res;
}


sub to_html { return shift->html }


##
# $self->validate();
# ------------------
# Validates this component. Returns TRUE if the component's data
# integrity is OK, FALSE otherwise.
##
sub validate
{
    my $self = shift;

    # set up the callback for errors
    local $MKDoc::Ouch::CALLBACK;
    $MKDoc::Ouch::CALLBACK = sub { $self->add_error (@_) };
    
    # return 1 if this html component is empty
    return 1 unless ($self->{data});
    
    my $nolinefeed_data = join '', split /\n|\r/, $self->{data};
    return if ($nolinefeed_data =~ /^\s+$/);
    
    my $ret = 1;
    eval {
	my $tidied_data = $self->html_tidy;
	$self->{data} = $tidied_data;
    };
    
    (defined $@ and $@) and do {
	new MKDoc::Ouch 'component/html/untidy';
	return;
    };
    
    return 1;
}


##
# $obj->html_tidy;
# ----------------
#   Return a clean version of the HTML.
##
sub html_tidy
{
    my $self   = shift;
    my $string = shift || $self->{data};
    
    my $cfg     = new MKDoc::Config ('GLOBAL');
    my $tidy    = MKDoc::Config->TIDY_COMMAND;  
    my $conf    = MKDoc::Config->TIDY_CONFIG;
    my $tmp_dir = MKDoc::Config->TEMP_DIR;
    
    my ($out, $err) = sysexec ("$tidy -config $conf", $tmp_dir, $string);
    Encode::_utf8_on ($out);
    die $err if ($? != 0 && $? != 256);
    
    # strip everything before <BODY> and after </BODY>
    $out =~ s/^.*<BODY[^>]*>\n?//si;
    $out =~ s\</BODY[^>]*>.*</HTML[^>]*>.*$\\si;
    return $out;
}


1;




